/********************************************************************************
	Title:    ADC.c
	Author:   Matthias Reis <mail@mreis.de>
	Date:     27.04.2004
	Purpose:  Analog-Digitalwandler Routinen fr Helligkeit und Temperatur fr die GrossUhr
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h>
#include "Types.h"
#include "Main.h"

u8 LDR=0;					//SpannungsWert, der am LDR gemessen wurde	
u16 NTC1=0;				// "                    NTC1
u16 NTC2=0;				// "					NTC2
u8 inMux=1;				//Gewhlter Eingabekanal (ADMUX), 1-LDR, 2-NTC1, 3-NTC2

/***************************************************************************************************

Function		: adcToTemp

Parameter In	: ADC der LDR 1,2
Parameter Out	: -

Description		: Rechnet den ADC-Wert der NTCs (10k, 10kSpannungsteiler9 um in Temperatur

Date			: MR 23.04.2004

****************************************************************************************************/
i16 adcToTemp(u16 in)
{
	return ((in-240)*94)/100;
}

/***************************************************************************************************

Function		: StartADC

Parameter In	: Kanal, 1-LDR, 2-NTC1. 3-NTC3
Parameter Out	: -

Description		: Startet den AD-Wndler und stzt doe Var inMux auf den gewhlten Kanal

Date			: MR 23.04.2004

****************************************************************************************************/
void StartADC(u8 in)
{
	//cbi(ADCSRA,ADEN);		//Einschalten ADC
	//Setze Multiplex/Kanal
	while (ADCSRA & _BV(ADSC)) {}		//Warte auf die letze Konvertierung
	inMux=in;
	switch (inMux)
	{
		case 1:
			ADMUX= 0;
			break;
		case 2:
			ADMUX= 1;
			break;
		case 3:	
			ADMUX= 2;
			break;
	}
	//sbi(ADCSRA,ADEN);		//Einschalten ADC
	ADCSRA |= _BV(ADSC);		//Starte Konvertierung
}