/********************************************************************************
	Title:    Programm fr die GrossUhr
	Author:   Matthias Reis <mail@mreis.de>
	Date:     05.05.2004
	Purpose:  Hauptschleife
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
	
	Programm fr die "GrossUhr".
	
	Verbindungen:
	==============
	*** OUT
	Display:
	SI-T/C/D<->	PB0
	CLK-C	<->	PD2
	G		<->	PD1
	CLK-T	<-> PD0
	LDR		<-> PC0/ADC0
	
	Day-Display:
	CLK-D	<->	PD7
	G-D		<->	PD6
	Row4	<->	PB1
	Row3	<->	PB2
	Row2	<->	PB3
	Row1	<->	PB4
	Row0	<->	PB5
	
	Analog:
	NTC1	<-> PC1/ADC1
	NTC2	<-> PC2/ADC2
	
	*** IN
	Tasten:
	T1		<->	PC5
	T2		<->	PC4
	T3		<->	PC3
	T4		<->	PD4
	T5		<->	PD5
	
	DCF77:
	Signal	<->	PD3/INT1
********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h>
#include "Types.h"
#include "Main.h"
#include "Display.h"
#include "ISR.h"
#include "Init.h"
#include "ADC.h"
#include "dcf77.h"
#include "clock.h"
#include "moon.h"
#include "Keys.h"
#include "eeprom.h"


u8 bright_hi=BRIGHT_LO;
BOOL flInit=FALSE;				//Ist die Uhr schon gestellt?
i8 bright_off=-1;				//Datum/Temperatur/Mond werden immer angezeigt
BOOL gFullDisplay=TRUE;			//Alles anzeigen


/***************************************************************************************************

Function		: delay

Parameter In	: in - Verzgerung
Parameter Out	: -

Description		: Primitive Verzgerung, nur fr Testzwecke

Date			: MR .04.2004

****************************************************************************************************/
void delay(u8 in)
{
	u8 i, j,k;
    
	for (i=0;i<in;i++) 
	{
		for (j=0;j<255;j++) 
		{ 
			for (k=0;k<255;k++); 
		}	
	}
}

/***************************************************************************************************

Function		: keylogic

Parameter In	: in - Gedrckte Taste 1-3
Parameter Out	: -

Description		: Implementiert die Tastenlogik

Date			: MR .04.2004

****************************************************************************************************/
void keylogic(u8 in)
{
	static u8 stage;
	static u8 substage;
	static u8 tmpWDay=255;		//WDay merken
	
	if (in==0xff)				//Timeout ist durchgelaufen, Abbrechen (wird in Main gesetzt)
		stage=4;
		
	if (tmpWDay==255)		
		tmpWDay=clock.wday;
		
	if (in==NEXT_KEY)
	{
		if (!substage)
			stage++;
		else
			substage++;
	}
	
	switch (stage)
	{
		case 1:		//Clock einstellen
			clock.wday=DISPL_CL;
			
			setTemp(clock.year,FALSE);
			displTemp();
			
			switch (substage)
			{
				case 0:		//10 Hour einstellen
					if (in==UP_KEY || in==DOWN_KEY)
					{
						substage=1;
						flInit=TRUE;	//Initialisierung durchgefhrt
						flBlink=FALSE;
					}
					break;
				
				case 1:		//1 Min einstellen
					if (clock.minute % 10 <9 && in==UP_KEY)
						clock.minute+=1;
					if (clock.minute % 10 >0 && in==DOWN_KEY)
						clock.minute-=1;
					break;
				
				case 2:		//10 Min einstellen
					if (clock.minute<50 && in==UP_KEY)
						clock.minute+=10;
					if (clock.minute>9 && in==DOWN_KEY)
						clock.minute-=10;
					break;
				
				case 3:		//1 Hour einstellen
					if (clock.hour % 10 <9 && in==UP_KEY)
						clock.hour+=1;
					if (clock.hour % 10 >0 && in==DOWN_KEY)
						clock.hour-=1;
					break;
				
				case 4:		//10 Hour einstellen
					if (clock.hour< 14 && in==UP_KEY)
						clock.hour+=10;
					if (clock.hour>9 && in==DOWN_KEY)
						clock.hour-=10;
					break;
				
				case 5:		//Year einstellen
					if (clock.year < 99 && in==UP_KEY)
						clock.year+=1;
					if (clock.year >1 && in==DOWN_KEY)
						clock.year-=1;
					
					setTemp(clock.year,FALSE);
					displTemp();
					break;
				
				case 6:		//Month einstellen
					if (clock.month <12 && in==UP_KEY)
						clock.month+=1;
					if (clock.month > 1 && in==DOWN_KEY)
						clock.month-=1;
					if (clock.day>mTage[clock.month])
						clock.day=mTage[clock.month];
					break;
				
				case 7:		//Day einstellen
					if (clock.day < mTage[clock.month] + (istSchaltjahr(clock.year) && clock.month==2 ? 1 : 0) && in==UP_KEY)
						clock.day+=1;
					if (clock.day >1 && in==DOWN_KEY)
						clock.day-=1;
					break;
				
				case 8:	//ende
					substage=0;
					break;
			}
			break;
		
		case 2:		//Helligkeit einstellen (Hi)
			clock.wday=DISPL_HI;
			if (bright_hi<0xff && in==UP_KEY)
				bright_hi+=1;
			if (bright_hi>0 && in==DOWN_KEY)
				bright_hi-=1;
				
			setTemp(bright_hi,FALSE);
			displTemp();
			break;
		
		/*case 3:		//Helligkeit einstellen (Lo)
			clock.wday=DISPL_LO;
			if (bright_lo<0xff && in==UP_KEY)
				bright_lo+=1;
			if (bright_lo>0 && in==DOWN_KEY)
				bright_lo-=1;
				
			setTemp(bright_lo,FALSE);
			displTemp();
			break;*/
		case 3:		//Ab welcher Helligkeit sollen Temp und Datum und Moon nicht mehr angezeigt werden?
			if (bright_off<127 && in==UP_KEY)
				bright_off+=1;
			if (bright_off>-1 && in==DOWN_KEY)
				bright_off-=1;
				
			setTemp(bright_off,FALSE);
			displTemp();

			break;
			
		case 4:		//Ende
			stage=0;
			substage=0;
			clock.wday=tmpWDay;
			tmpWDay=255;
			//Sichern der Werte im eeprom
			Save();
			break;
	}
	setMoon();	//Mond aktualisieren
	setDay();	//Wochentag aktualisieren
}

/***************************************************************************************************

Function		: main

Parameter In	: -
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/

int main(void)
{
	u8 key;							//Welche Taste gedrckt
	BOOL Reset=TRUE;				//Fr TimeOut bei Tasten
	i8 OffTime=OFFTIME;				//Wenn es dunkel wird (LDR<=bright_off) soll nach OFFTIME-Minuten die Anzeige auf "nur Uhrzeit" springen
	
	//Presets aus eeprom laden
	Load();
	init();
	EraClock();
	EraTemp();
	
	clock.month=1;
	clock.day=1;
	clock.minute=0;
	clock.hour=0;
	clock.year=4;
	setMoon();						//Mondstellung ermitteln
	setDay();
	
	setClock();						//Daten in das Clock-Array schieben
	displClock();					//Display Clock
	
	sei();							//Enable Ints
	
	//Mainloop
	for (;;)
	{
		scan_dcf77();				// receive dcf77
		
		WDayInt();					//Anzeige des Wochentags, Multiplexed
		
		if (flClockRefresh)			//flag zur Anzeige der Uhr
		{
			//Was soll angezeigt werden?
			if (LDR>bright_off)		//Es ist hell, also Anzeige nicht reduzieren auf Uhrzeit
				OffTime=OFFTIME;
			else					//Es ist dunkel, nach OFFTIME Minuten Anzeige reduzieren
				OffTime--;
			
			if (OffTime<=0)
			{
				gFullDisplay=FALSE;	//Anzeige nur noch Uhrzeit
				OffTime=0;
			}
			else
				gFullDisplay=TRUE;	//Anzeige alles
				
			setClock();
			displClock();
			flClockRefresh=0;
		}
		
		if (flADCRead)				//flag zum erneuten einlesen der Sensoren ber ADC
		{
			
			//setTemp(clock.second,0);
			
			if (Reset)		//Keine Tastatureingabe (Temp wird dann anderweitig gebraucht)
			{
				if (gTempSwitch>TEMP_SWITCHTIME*2)
					gTempSwitch=0;
				
				if (gTempSwitch<=TEMP_SWITCHTIME)
				{
					StartADC(3);	//NTC2
					setTemp(adcToTemp(NTC2),TRUE);
				}
				else
				{
					StartADC(2);	//NTC1
					setTemp(adcToTemp(NTC1),FALSE);
				}
				
				//setTemp(dcf77_pulse,0);
				//setTemp(LDR,TRUE);
				displTemp();
			}
			
			StartADC(1);	//LDR
			
			flADCRead=0;
		}

		if (flOneSec)				//flag fr: eine Sekunde ist vergangen
		{
			incSec();				//Uhrzeit um 1s erhhen
			flOneSec=0;
		}
		
		key=KeyPressed();
		
		if (key)
		{	
			gTimeout=TIMEOUT;				//x Sekunden Timeout
			keylogic(key);
			setClock();
			displClock();
			//clock.wday=DISPL_CL;
			Reset=FALSE;
		}
	
		if (!gTimeout && !Reset)	//TimeOut ist 0 und Reset auch
		{
			keylogic(0xff);			//Anzeige nach Timeout zurcksetzen 
			Reset=TRUE;
		}
	}
	
	//Wird nie erreicht
	return 0;
	
}


/***************************************************************************************************

Function		: 

Parameter In	: -
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/
