/********************************************************************************
	Title	: Main.h
	Author	: Matthias Reis <mail@mreis.de>
	Date	: 27.04.2004
	Purpose	: Header mit defines und Globalen Variablen
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
	Fuses	: CKSEL1 + CKSEL2 = 0 (Set!) = 8MHz Takt
			  CKOPT=1 (Set!) = Capacitors on TOSC1/2 activated
********************************************************************************/


#ifndef Main_H
#define Main_H

/*
** Macros and constants
*/

#define G PIND1						//Steuerung der Helligkeit (ber |G am HC595)
#define G_D PIND6					//Steuerung der Helligkeit der Wochentagsanzeige (ber |G am HC595)
                            		
#define SI PINB0					//SI fr alle HC595 Ketten (Clock, Temperatur, Wochentag)
#define CLK_C PIND2					//Clk (RCK und TCK) fr die Clock-Kette
#define CLK_T PIND0					//Clk (RCK und TCK) fr die Temperatur-Kette
#define CLK_D PIND7					//Clk (RCK und TCK) fr Wochentag
#define DCF77 PIND3					//Anschluss des DCF77-Moduls
                            		
#define T1 PINC5					//Taste 1 (UP)
#define T2 PINC4					//Taste 2 (DOWN)
#define T3 PINC3					//Taste 3 (NEXT)
                            		
//#define QUARZ 128					//TCNT2 (Quarz 32768 Hz, Prescaler 256 = 1sec.)
#define COUNTER 215					//TCNT0, bei einem prescaler von 8 ist das  8MHz/8/50= 50 us (* 255 ~ 12.7 ms)
                            		
#define BRIGHT_HI 220				//Obergrenze fr Helligkeit
#define BRIGHT_LO 0					//Untergrenze fr Helligkeit
                            		
#define TIMEOUT 10					//Abbruch der Eingabe nach x Sekunden
#define TEMP_SWITCHTIME	4			//Umschalten der Temperatur (auen/innen) alle x Sekunden 

#define OFFTIME 5					//Nach wieviel Minuten Dunkelheit soll nur noch die Uhrzeit angezeigt werden?

/*
** Globale Variablen
*/

volatile u8 bright;					//Helligkeit des Displays

//Flags
volatile BOOL flClockRefresh;		//Signal zum Refresh des Clock-Displays
volatile BOOL flTempRefresh;		//Signal zum Refresh des Temperatur-Displays
volatile BOOL flADCRead;			//Signal zum neueinlesen der Sensoren (jede Sekunde)
volatile BOOL flOneSec;				//Signal: Eine Sekunde ist vergangen
//Tastatur
volatile BOOL flKeyLatency;			//Latenzzeit zum Starten der Repeatkey-Funktionalitt
volatile BOOL flKeyReset;			//Keyreset bewirkt ein Erkennen einer gedrckten Taste

volatile u8 gTimeout;				//TimeoutVariable fr Keys
volatile u8 gTempSwitch;			//Umschaltung zw. Internem und externem Temp-Sensor

volatile u8 flBlink;				//Blinken der Uhr wenn nicht initialisiert
extern BOOL flInit;					//Ist die Uhr schon gestellt?

extern u8 bright_hi;				//Einstellung der Helligkeit der Anzeige

extern i8 bright_off;				//Dunkelheit, die erreicht sein mu, damit nur noch die Uhrzeit angezeigt wird (-1 bedeutet immer alles anzeigen)
extern BOOL gFullDisplay;			//Soll alles angezeigt werden, oder nur Uhrzeit (abhngig von der Helligkeit)

#endif